# SPDX-License-Identifier: MIT
# ISO 693-1 language codes from pycountry
# This file is automatically generated, DO NOT EDIT!
Languages = {
    'de': 'German',
    'en': 'English',
    'es': 'Spanish',
    'fi': 'Finnish',
    'fr': 'French',
    'it': 'Italian',
    'ja': 'Japanese',
    'pt': 'Portuguese',
}
